
__doc__='''ExternalTestCase

A class for running external test cases writen for/with pyunit
'''

from Shared.Hiper.voodoo import CallableItem, tabs
import Globals
from AccessControl import ClassSecurityInfo
import unittest
from DocumentTemplate.DT_Util import InstanceDict
from Tests import exc_info2str

class TestResult(unittest.TestResult):
    "TestResult for use with TestMonitor and ExternalTestCase"

    def __init__(self, exTestCase, testMonitor):
	unittest.TestResult.__init__(self)
	self.monitor = testMonitor
        self.extc = exTestCase

    #testString = Globals.HTMLFile('Products/ZUnit/dtml/ExternaTestCase/testString')
    def testString(self, test):
        return ("ExternalTestCase: %s, around %s, running %s" %
                (self.extc.id, self.extc.instantiator, `test`))

    def addError(self, test, err):
	unittest.TestResult.addError(self, test, err)
	self.monitor.addError(self.testString(test), exc_info2str(err))

    def addFailure(self, test, err):
	unittest.TestResult.addFailure(self, test, err)
	self.monitor.addFailure(self.testString(test), exc_info2str(err))

    def stopTest(self, test):
        unittest.TestResult.stopTest(self, test)
        self.monitor.stopTest(test)

    def startTest(self, test):
        unittest.TestResult.startTest(self, test)
        if hasattr(test, 'setZopeContext'):
            # we can afford to send the test itself here `cause it has a
            # sensible id() implementation (I hope...)
            context = self.monitor.startTestWithContext(test)
            # set the context for tests that expect it
            # (ZUnit 0.1 tests, for instance)
            test.setZopeContext(context)
        else:
            self.monitor.startTest(test)


class ExternalTestCase(CallableItem):
    name = 'ExternalTestCase'
    id = 'ExternalTestCase'
    meta_type = 'ZUnit External Test Case'
    icon = None # for the time being

    security = ClassSecurityInfo()
    security.declareObjectProtected("View Test Cases")

    # the tabs
    manage_options = (({'label':'Edit', 'action':'manage_main'},) +
                      tabs.base + tabs.view)

    def __init__(self, id, title, instantiator):
        self.id = id
        self.title = title
        self.instantiator = instantiator

    security.declarePrivate("_testInstance")
    def _testInstance(self):
        test = unittest.createTestInstance(self.instantiator)
        return test

    security.declareProtected('View Test Cases', 'render')
    def render(self, _):
        "replace with something more meaningful"
        return self.countTestCases()

    security.declareProtected('Run Test Cases', 'run')
    def run(self, testMonitor):
        tr = TestResult(self, testMonitor)
        self._testInstance().run(tr)

    security.declareProtected('View Test Cases', 'countTests')
    def countTests(self):
        "show the number of tests"
        return self._testInstance().countTestCases()

    security.declareProtected('View Test Cases', 'manage_main')
    manage_main=Globals.HTMLFile('dtml/extcEdit', globals())

    security.declareProtected('Change Test Cases', 'manage_edit')
    def manage_edit(self, title='', instantiator='', REQUEST=None):
        """Edit ExternalTestCase properties."""
        self.title = title
        self.instantiator = instantiator
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('manage_main?manage_tabs_message=Updated.')

Globals.InitializeClass(ExternalTestCase)

def manage_addExternalTestCase(container, id, REQUEST=None):
    '''Add an ExternalTestCase to container'''

    extc = ExternalTestCase(id,
                            title=REQUEST['title'],
                            instantiator=REQUEST['instatiator']
                            )
    container._setObject(id, extc)

    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(REQUEST['URL1']+'/manage_main')
    else:
        return extc

manage_addExternalTestCaseForm=Globals.HTMLFile('dtml/addExternalTestCaseForm',
                                                globals())
