
__doc__='''TestMonitor

Class and supporting funcions for capturing the results of test runs.
'''

import Globals
import string

# Our Neighbours
from TestReport import TestReport

class TestMonitor:
    """Our closest equivalent to unittest.TestResult.
    Instantiated by TestRunner to store values in a TestReport"""


    # standard security thingy
    #security = ClassSecurityInfo()
    #security.declareObjectProtected("View Test Cases")

    
    def __init__(self, testRunner, REQUEST=None):
        # real id will be given at close()
        if REQUEST is not None:
            self.REQUEST = REQUEST
            self.RESPONSE = REQUEST.RESPONSE
        else:
            self.REQUEST = {}
            self.RESPONSE = None
        # FIXME: find a better way to store the evolution of the test
        # than the rendered result
        self.resultList = []
        # things taken from unittest.TestResult
        self.failures = []
        self.errors = []
        self.testsRun = 0
        # I'd like to have an use for this variable below,
        # but first I'd have to figure out a way to check if the user
        # has pressed the stop button on their browser (or maybe even a
        # stop button on the running page), which would cause the connection
        # to the browser to break up, but I don't think RESPONSE knows about
        # this. hmm... seems like a nice hacking project...
        self.shouldStop = 0
        # well, now I know: RESPONSE definetly doesn't know about this.
        # I think I know a way to work around it, but it will require
        # dark grey magic: wrap around Response.stdout to check if its
        # channel is open on write and raise an exception if it isn't
        self.toReport = None
        # begin writing about tests
        self._display(self.allStart(self.REQUEST))
        # now we create the test report
        id = testRunner.ZopeTime().strftime('%Y%m%d_%H%M%S')
        self.testReport = TestReport(id, self)
        testRunner._setObject(id, self.testReport)
        

    def _display(self, htstring):
        # store the rendered message
        self.resultList.append(htstring)
        if self.RESPONSE:
            try:
                self.RESPONSE.write(htstring)
            except IOError:
                self.stop()
                self.RESPONSE = None

    def addError(self, test, error):
        "called when something has gone wrong in the testing process"
	self.errors.append((test, error))
        self.toReport = self.testError(self.REQUEST, test=test, error=error)

    def addFailure(self, test, error):
        "called when the test has officially failed"
	self.failures.append((test, error))
	self.toReport = self.testFailure(self.REQUEST, test=test, error=error)

    def stopTest(self, test):
        self._display(self.toReport) # report the status of the last test
	htstring = self.testFinish(self.REQUEST, test=test)
	self._display(htstring)
 
    def startTest(self, test):
        "Called when a test is about to start"
        self.testsRun = self.testsRun + 1
	htstring = self.testStart(self.REQUEST, test=test)
	self._display(htstring)
        self.toReport = self.testSuccess(self.REQUEST, test=test)

    def startTestWithContext(self, test):
        "called when a test is about to start and it wants a context"
        self.startTest(test)
        return self.testReport.newContext(test.id())

    def stop(self):
        "Indicates that the tests should be aborted"
        #we need to rethink ways to notify tests that they should stop
        self.shouldStop = 1

    def finish(self):
        keep_data = self.REQUEST.get('keep_data', 0)
        self._display(self.allFinish(self.REQUEST))
        self.testReport.finish(keep_data)

    def __repr__(self):
        return "<%s run=%i errors=%i failures=%i>" % \
               (self.__class__, self.testsRun, len(self.errors),
                len(self.failures))

    # the dtml below should reference the resultList property with a
    # <dtml-in> tag
    render = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/index_html')
    
    # html parts that will be streammed for the user
    testError = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/error')
    testSuccess = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/success')
    testFailure = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/failure')
    testFinish = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/finish')
    testStart = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/start')
    allStart = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/allStart')
    allFinish = Globals.HTMLFile('Products/ZUnit/dtml/TestMonitor/allFinish')
