
__doc__='''TestReport

Class and supporting funcions for capturing the results of test runs.
'''

from Shared.Hiper.voodoo import CallableItem, tabs
import Globals
from AccessControl import ClassSecurityInfo
from OFS.ObjectManager import ObjectManager
import string
import traceback

class TestReport(ObjectManager, CallableItem):
    """A TestResult that double as a test report.
    Instantiated by TestRunner to store values in a TestReport"""

    # Zope class attributes
    name = 'TestReport'
    id = 'TestReport'
    meta_type = 'ZUnit Test Report'
    icon = None
    all_meta_types = ()

    manage_options = (tabs.contents + tabs.base +
                      ({'label':'View', 'action':'publish'},))

    # standard security thingy
    security = ClassSecurityInfo()
    security.declareObjectProtected("View Test Cases")
    
    def __init__(self, id, monitor):
        CallableItem.__init__(self, id)
        self._v_monitor = monitor

    def finish(self, keep_data=1):
        monitor = self._v_monitor
        # FIXME: find a better way to store the evolution of the test
        # than the rendered result
        self.resultList = monitor.resultList
        # things taken from unittest.TestResult
        self.failures = monitor.failures
        self.errors = monitor.errors
        self.testsRun = monitor.testsRun
        # delete uneeded transients
        del self._v_monitor
        if not keep_data:
            self.manage_delObjects(self.objectIds())
        if self.wasSuccessful():
            self.title = 'Successful test report'
        else:
            self.title = 'Failed test report'

    security.declareProtected("View Test Cases", 'wasSuccessful')
    def wasSuccessful(self):
        "Tells whether or not this result was a success"
        return len(self.failures) == len(self.errors) == 0

    security.declareProtected("View Test Cases", '__repr__')
    def __repr__(self):
        return "<%s run=%i errors=%i failures=%i>" % \
               (self.__class__, self.testsRun, len(self.errors),
                len(self.failures))

    def newContext(self, testname):
        # five digits in case you want to run the C3 test suite
        # or something. Yea right.
        id = "%05d_%s" % (self._v_monitor.testsRun, testname)
        self.manage_addFolder(id)
        return getattr(self, id)

    security.declareProtected('View Test Cases', 'publish')
    publish = Globals.HTMLFile('Products/ZUnit/dtml/TestReport/publish')

    # the dtml below should reference the resultList property with a
    # <dtml-in> tag
    security.declareProtected('View Test Cases', 'render')
    render = Globals.HTMLFile('Products/ZUnit/dtml/TestReport/render')

Globals.InitializeClass(TestReport)
