
__doc__='''ExternalTestCase

A class for running external test cases writen for/with pyunit
'''

from Shared.Hiper import voodoo
from OFS import ObjectManager, PropertyManager
import Globals
from DocumentTemplate import html_quote
from DocumentTemplate.DT_Var import newline_to_br

from AccessControl import ClassSecurityInfo
from TestMonitor import TestMonitor
from Tests import exc_info2str


class TestRunner(ObjectManager.ObjectManager,
                 voodoo.CallableItem,
                 PropertyManager.PropertyManager):
    """TestRunner class
    """
    __init__ = voodoo.CallableItem.__init__
    __setstate__ = voodoo.CallableItem.__setstate__

    #################################################
    ### Zope class attributes

    name = 'TestRunner'
    id = 'TestRunner'
    meta_type = 'ZUnit TestRunner'
    icon = None
    isAnObjectManager=1
    isPrincipiaFolderish=1

    manage_options = (voodoo.tabs.contents +
                      ({'label':'Run Tests', 'action':'manage_RunTestsForm',
                        'help':('ZUnit', 'RunTests.stx')},) +
                      voodoo.tabs.properties +
                      voodoo.tabs.base +
                      voodoo.tabs.find)

    # things voodoo magically do for us
    _prototype_attrs = {'test_ids' : [],
                        'title':'All Your Base Are Belong to Us'}

    #################################################
    ## Misc ObjectManager stuff
    all_meta_types = ()

    _properties = ({'id': 'title', 'type': 'string', 'mode': 'w'},
                   {'id': 'test_ids', 'type': 'lines', 'mode': 'w'},)

    security = ClassSecurityInfo()
    security.declareObjectProtected('View Test Cases')

    security.declareProtected('View Test Cases', 'testList')
    def testList(self):
        # get empty lines out
        retval = filter(None, self.test_ids)
        return map(self.restrictedTraverse, retval)
        #return retval
    
    security.declareProtected('View Test Cases', 'countTests')
    def countTests(self):
        "Return the number of individual tests"
        total = 0
        for suite in self.testList():
            total = total + suite.countTests()
        return total
    
    security.declareProtected('Run Test Cases', 'running')
    def running(self, REQUEST=None):
        "Actually run the tests"
        REQUEST.RESPONSE.setHeader("content-type", "text/html")
        try:
            REQUEST.RESPONSE.write(self.standard_html_header(self, REQUEST))
            tm = TestMonitor(self, REQUEST)
            tl = self.testList()
            #REQUEST.RESPONSE.write(self.testsStarting(REQUEST,
            #                                          startTime = before))
            for test in tl:
                test.run(tm)
            tm.finish()
            #REQUEST.RESPONSE.write(self.testsFinished(REQUEST,
            #                                          endTime = after))
            #REQUEST.RESPONSE.write("<hr>\ntr: %s <hr>\n me: %s<hr>\n" %
                                    #(html_quote(getattr(self, id).__dict__),
                                      #html_quote(self.__dict__)))

        except:
            errstr = exc_info2str()
            print "ZUnit: Error during streaming:\n" + errstr
            REQUEST.RESPONSE.write("<H1>Error during streaming:</H1>")
            REQUEST.RESPONSE.write(errstr)
        REQUEST.RESPONSE.write(self.standard_html_footer(self, REQUEST))

    security.declareProtected('View Test Cases', 'manage_RunTestsForm')
    manage_RunTestsForm = Globals.HTMLFile('Products/ZUnit/dtml/TestRunner/RunTests')

    security.declareProtected('Know the secrets of life', 'manage_dict')
    def manage_dict(self, REQUEST=None):
        "debug it"
        return `self.__dict__`

Globals.InitializeClass(TestRunner)

# constructors
def manage_addTestRunner(container, id, REQUEST=None):
    '''Add a TestRunner to container'''

    tr = TestRunner(id)
    container._setObject(id, tr)

    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(REQUEST['URL1']+'/manage_main')
    else:
        return rt

manage_addTestRunnerForm=Globals.HTMLFile('dtml/addTestRunnerForm',
                                                globals())
